@@ -3,7 +3,8 @@ from groclient.lib import get_default_logger
 
 
 logger = get_default_logger()
-logger.warning('''
+logger.warning(
+    """
     Deprecation Warning!
 
     You are importing modules from deprecated `api` module to access Gro
@@ -14,4 +15,5 @@ logger.warning('''
        with: groclient.cfg
 
     Please reach out to api-support@gro-intelligence.com if you need any help!
-''')
+"""
+)
