@@ -4,7 +4,8 @@ from groclient.lib import get_default_logger
 
 
 logger = get_default_logger()
-logger.warning('''
+logger.warning(
+    """
     Deprecation Warning!
 
     You are importing modules from deprecated `api` module to access Gro
@@ -15,4 +16,5 @@ logger.warning('''
        with: groclient
 
     Please reach out to api-support@gro-intelligence.com if you need any help!
-''')
+"""
+)
