@@ -1,7 +1,8 @@
 from groclient.lib import *
 
 logger = get_default_logger()
-logger.warning('''
+logger.warning(
+    """
     Deprecation Warning!
 
     You are importing modules from deprecated `api` module to access Gro
@@ -12,4 +13,5 @@ logger.warning('''
        with: groclient.lib
 
     Please reach out to api-support@gro-intelligence.com if you need any help!
-''')
+"""
+)
